@@ -396,6 +396,7 @@ class Api:
         timeout:      Optional[int]  = None,
         filter:       _FilterT       = None,
         full_state:   Optional[bool] = None,
+        set_presence: Optional[str]  = None,
     ):
         # type: (...) -> Tuple[str, str]
         """Synchronise the client's state with the latest state on the server.
@@ -408,13 +409,20 @@ class Api:
                 to.
             timeout (int): The maximum time to wait, in milliseconds, before
                 returning this request.
-            sync_filter (Union[None, str, Dict[Any, Any]):
+            filter (Union[None, str, Dict[Any, Any]):
                 A filter ID or dict that should be used for this sync request.
             full_state (bool, optional): Controls whether to include the full
                 state for all rooms the user is a member of. If this is set to
                 true, then all state events will be returned, even if since is
                 non-empty. The timeline will still be limited by the since
                 parameter.
+            set_presence (str, optinal): Controls whether the client is automatically
+                marked as online by polling this API. If this parameter is omitted
+                then the client is automatically marked as online when it uses this API.
+                Otherwise if the parameter is set to "offline" then the client is not
+                marked as being online when it uses this API. When set to "unavailable",
+                the client is marked as being idle.
+                One of: ["offline", "online", "unavailable"]
         """
         query_parameters = {"access_token": access_token}
 
@@ -427,6 +435,9 @@ class Api:
         if timeout is not None:
             query_parameters["timeout"] = str(timeout)
 
+        if set_presence:
+            query_parameters["set_presence"] = set_presence
+
         if isinstance(filter, dict):
             filter_json = json.dumps(filter, separators=(",", ":"))
             query_parameters["filter"] = filter_json
@@ -469,6 +480,32 @@ class Api:
             Api.to_json(body)
         )
 
+    @staticmethod
+    def room_get_event(
+        access_token: str,
+        room_id: str,
+        event_id: str
+    ) -> Tuple[str, str]:
+        """Get a single event based on roomId/eventId.
+
+        Returns the HTTP method and HTTP path for the request.
+
+        Args:
+            access_token (str): The access token to be used with the request.
+            room_id (str): The room id of the room where the event is in.
+            event_id (str): The event id to get.
+        """
+        query_parameters = {"access_token": access_token}
+
+        path = "rooms/{room}/event/{event_id}".format(
+            room=room_id, event_id=event_id
+        )
+
+        return (
+            "GET",
+            Api._build_path(path, query_parameters)
+        )
+
     @staticmethod
     def room_put_state(
         access_token, # type str
@@ -1210,6 +1247,30 @@ class Api:
             Api.to_json(content)
         )
 
+    @staticmethod
+    def update_receipt_marker(
+        access_token: str,
+        room_id: str,
+        event_id: str,
+        receipt_type: str = "m.read",
+    ) -> Tuple[str, str]:
+        """Update the marker of given `receipt_type` to specified `event_id`.
+
+        Returns the HTTP method and HTTP path for the request.
+
+        Args:
+            access_token (str): The access token to be used with the request.
+            room_id (str): Room id of the room where the marker should
+                be updated
+            event_id (str): The event ID the read marker should be located at
+            receipt_type (str): The type of receipt to send. Currently, only
+                `m.read` is supported by the Matrix specification.
+        """
+        query_parameters = {"access_token": access_token}
+        path = f"rooms/{room_id}/receipt/{receipt_type}/{event_id}"
+
+        return ("POST", Api._build_path(path, query_parameters))
+
     @staticmethod
     def room_read_markers(
         access_token,       # type: str
@@ -1218,7 +1279,7 @@ class Api:
         read_event=None,    # type: Optional[str]
     ):
         # type: (...) -> Tuple[str, str, str]
-        """Update read markers for a room.
+        """Update fully read marker and optionally read marker for a room.
 
         This sets the position of the read marker for a given room,
         and optionally the read receipt's location.
@@ -1227,7 +1288,7 @@ class Api:
 
         Args:
             access_token (str): The access token to be used with the request.
-            room_id (str): Room id of the room of the room where the read
+            room_id (str): Room id of the room where the read
                 markers should be updated
             fully_read_event (str): The event ID the read marker should be
                 located at.
@@ -1457,6 +1518,48 @@ class Api:
             Api.to_json(content)
         )
 
+    @staticmethod
+    def get_presence(access_token: str, user_id: str) -> Tuple[str, str]:
+        """Get the given user's presence state.
+
+        Returns the HTTP method and HTTP path for the request.
+
+        Args:
+            access_token (str): The access token to be used with the request.
+            user_id (str): User id whose presence state to get.
+        """
+        query_parameters = {"access_token": access_token}
+        path = "presence/{user_id}/status".format(user_id=user_id)
+
+        return (
+            "GET",
+            Api._build_path(path, query_parameters),
+        )
+
+    @staticmethod
+    def set_presence(access_token: str, user_id: str, presence: str, status_msg: str = None):
+        """This API sets the given user's presence state.
+
+        Returns the HTTP method, HTTP path and data for the request.
+
+        Args:
+            access_token (str): The access token to be used with the request.
+            user_id (str): User id whose presence state to get.
+            presence (str): The new presence state.
+            status_msg (str, optional): The status message to attach to this state.
+        """
+        query_parameters = {"access_token": access_token}
+        content = {"presence": presence}
+        if status_msg:
+            content["status_msg"] = status_msg
+        path = "presence/{user_id}/status".format(user_id=user_id)
+
+        return (
+            "PUT",
+            Api._build_path(path, query_parameters),
+            Api.to_json(content)
+        )
+
     @staticmethod
     def whoami(access_token):
         # type (str) -> Tuple[str, str]
