@@ -34,9 +34,11 @@ from collections import defaultdict
 
 from ..crypto import ENCRYPTION_ENABLED
 from ..events import (
+    AccountDataEvent,
     BadEventType,
     BadEvent,
     UnknownBadEvent,
+    EphemeralEvent,
     Event,
     MegolmEvent,
     RoomEncryptionEvent,
@@ -44,8 +46,9 @@ from ..events import (
     ToDeviceEvent,
     RoomKeyRequest,
     RoomKeyRequestCancellation,
+    PresenceEvent,
 )
-from ..exceptions import LocalProtocolError, MembersSyncError
+from ..exceptions import LocalProtocolError, MembersSyncError, EncryptionError
 from ..log import logger_group
 from ..responses import (
     ErrorResponse,
@@ -63,10 +66,12 @@ from ..responses import (
     RoomForgetResponse,
     RoomKeyRequestResponse,
     RoomMessagesResponse,
+    RoomGetEventResponse,
     ShareGroupSessionResponse,
     SyncResponse,
     SyncType,
     ToDeviceResponse,
+    PresenceGetResponse,
 )
 from ..rooms import MatrixInvitedRoom, MatrixRoom
 
@@ -118,7 +123,7 @@ class ClientCallback:
     """nio internal callback class."""
 
     func: Callable = field()
-    filter: Union[Tuple[Type], Type, None] = None
+    filter: Union[Tuple[Type, ...], Type, None] = None
 
 
 @dataclass(frozen=True)
@@ -156,7 +161,7 @@ class ClientConfig:
             raise ImportWarning(
                 "Encryption is enabled in the client "
                 "configuration but dependencies for E2E "
-                "encrytpion aren't installed."
+                "encryption aren't installed."
             )
 
 
@@ -214,6 +219,8 @@ class Client:
         self.event_callbacks: List[ClientCallback] = []
         self.ephemeral_callbacks: List[ClientCallback] = []
         self.to_device_callbacks: List[ClientCallback] = []
+        self.presence_callbacks: List[ClientCallback] = []
+        self.room_account_data_callbacks: List[ClientCallback] = []
 
     @property
     def logged_in(self) -> bool:
@@ -692,6 +699,9 @@ class Client:
         if join_info.summary:
             room.update_summary(join_info.summary)
 
+        if join_info.unread_notifications:
+            room.update_unread_notifications(join_info.unread_notifications)
+
     def _handle_timeline_event(
         self,
         event: Union[Event, BadEventType],
@@ -756,6 +766,13 @@ class Client:
                     if cb.filter is None or isinstance(event, cb.filter):
                         cb.func(room, event)
 
+            for event in join_info.account_data:
+                room.handle_account_data(event)
+
+                for cb in self.room_account_data_callbacks:
+                    if cb.filter is None or isinstance(event, cb.filter):
+                        cb.func(room, event)
+
             if room.encrypted and self.olm is not None:
                 self.olm.update_tracked_users(room)
 
@@ -764,6 +781,21 @@ class Client:
         if self.store:
             self.store.save_encrypted_rooms(encrypted_rooms)
 
+    def _handle_presence_events(self, response: SyncType):
+        for event in response.presence_events:
+            for room_id in self.rooms.keys():
+                if event.user_id not in self.rooms[room_id].users:
+                    continue
+
+                self.rooms[room_id].users[event.user_id].presence = event.presence
+                self.rooms[room_id].users[event.user_id].last_active_ago = event.last_active_ago
+                self.rooms[room_id].users[event.user_id].currently_active = event.currently_active
+                self.rooms[room_id].users[event.user_id].status_msg = event.status_msg
+
+            for cb in self.presence_callbacks:
+                if cb.filter is None or isinstance(event, cb.filter):
+                    cb.func(event)
+
     def _handle_expired_verifications(self):
         expired_verifications = self.olm.clear_verifications()
 
@@ -817,6 +849,8 @@ class Client:
 
         self._handle_joined_rooms(response)
 
+        self._handle_presence_events(response)
+
         if self.olm:
             self._handle_expired_verifications()
             self._handle_olm_events(response)
@@ -942,6 +976,16 @@ class Client:
         elif response.room_id in self.invited_rooms:
             del self.invited_rooms[response.room_id]
 
+    def _handle_presence_response(self, response: PresenceGetResponse):
+        for room_id in self.rooms.keys():
+            if response.user_id not in self.rooms[room_id].users:
+                continue
+
+            self.rooms[room_id].users[response.user_id].presence = response.presence
+            self.rooms[room_id].users[response.user_id].last_active_ago = response.last_active_ago
+            self.rooms[room_id].users[response.user_id].currently_active = response.currently_active or False
+            self.rooms[room_id].users[response.user_id].status_msg = response.status_msg
+
     def receive_response(
         self, response: Response
     ) -> Union[None, Coroutine[Any, Any, None]]:
@@ -985,6 +1029,14 @@ class Client:
             self._handle_room_forget_response(response)
         elif isinstance(response, ToDeviceResponse):
             self._handle_olm_response(response)
+        elif isinstance(response, RoomGetEventResponse):
+            if isinstance(response.event, MegolmEvent) and self.olm is not None:
+                try:
+                    response.event = self.decrypt_event(response.event)
+                except EncryptionError:
+                    pass
+        elif isinstance(response, PresenceGetResponse):
+            self._handle_presence_response(response)
         elif isinstance(response, ErrorResponse):
             if response.soft_logout:
                 self.access_token = ""
@@ -1124,8 +1176,8 @@ class Client:
     def add_event_callback(
         self,
         callback: Callable[[MatrixRoom, Event], None],
-        filter: Union[Type, Tuple[Type]],
-    ):
+        filter: Union[Type[Event], Tuple[Type[Event], ...]],
+    ) -> None:
         """Add a callback that will be executed on room events.
 
         The callback can be used on joined rooms as well as on invited rooms.
@@ -1136,7 +1188,9 @@ class Client:
             callback (Callable[[MatrixRoom, Event], None]): A
                 function that will be called if the event type in the filter
                 argument is found in a room timeline.
-            filter (Union[Type, Tuple[Type]]): The event type or a tuple
+
+            filter (Union[Type[Event], Tuple[Type[Event], ...]]):
+                The event type or a tuple
                 containing multiple types for which the function will be
                 called.
 
@@ -1146,34 +1200,67 @@ class Client:
 
     def add_ephemeral_callback(
         self,
-        callback: Callable[[MatrixRoom, Event], None],
-        filter: Union[Type, Tuple[Type]],
-    ):
+        callback: Callable[[MatrixRoom, EphemeralEvent], None],
+        filter: Union[Type[EphemeralEvent], Tuple[Type[EphemeralEvent], ...]],
+    ) -> None:
         """Add a callback that will be executed on ephemeral room events.
 
         Args:
-            callback (Callable[MatrixRoom, Event]): A function that will be
+            callback (Callable[MatrixRoom, EphemeralEvent]):
+                A function that will be
                 called if the event type in the filter argument is found in the
                 ephemeral room event list.
-            filter (Type, Tuple[Type]): The event type or a tuple containing
+
+            filter
+            (Union[Type[EphemeralEvent], Tuple[Type[EphemeralEvent], ...]]):
+                The event type or a tuple containing
                 multiple types for which the function will be called.
 
         """
         cb = ClientCallback(callback, filter)
         self.ephemeral_callbacks.append(cb)
 
+    def add_room_account_data_callback(
+        self,
+        callback: Callable[[MatrixRoom, AccountDataEvent], None],
+        filter: Union[
+            Type[AccountDataEvent],
+            Tuple[Type[AccountDataEvent], ...],
+        ],
+    ) -> None:
+        """Add a callback that will be executed on room account data events.
+
+        Args:
+            callback (Callable[[MatrixRoom, ToDeviceEvent], None]):
+                A function that will be
+                called if the event type in the filter argument is found in
+                the room account data event list.
+
+            filter
+            (Union[Type[AccountDataEvent], Tuple[Type[AccountDataEvent, ...]]):
+                The event type or a tuple
+                containing multiple types for which the function
+                will be called.
+
+        """
+        cb = ClientCallback(callback, filter)
+        self.room_account_data_callbacks.append(cb)
+
     def add_to_device_callback(
         self,
         callback: Callable[[ToDeviceEvent], None],
-        filter: Union[Type, Tuple[Type]],
-    ):
+        filter: Union[Type[ToDeviceEvent], Tuple[Type[ToDeviceEvent], ...]],
+    ) -> None:
         """Add a callback that will be executed on to-device events.
 
         Args:
             callback (Callable[[ToDeviceEvent], None]): A function that will be
                 called if the event type in the filter argument is found in a
                 the to-device part of the sync response.
-            filter (Union[Type, Tuple[Type]]): The event type or a tuple
+
+            filter
+            (Union[Type[ToDeviceEvent], Tuple[Type[ToDeviceEvent], ...]]):
+                The event type or a tuple
                 containing multiple types for which the function
                 will be called.
 
@@ -1181,6 +1268,25 @@ class Client:
         cb = ClientCallback(callback, filter)
         self.to_device_callbacks.append(cb)
 
+    def add_presence_callback(
+        self,
+        callback: Callable[[PresenceEvent], None],
+        filter: Union[Type, Tuple[Type]],
+    ):
+        """Add a callback that will be executed on presence events.
+
+        Args:
+            callback (Callable[[PresenceEvent], None]): A function that will be
+                called if the event type in the filter argument is found in a
+                the presence part of the sync response.
+            filter (Union[Type, Tuple[Type]]): The event type or a tuple
+                containing multiple types for which the function
+                will be called.
+
+        """
+        cb = ClientCallback(callback, filter)
+        self.presence_callbacks.append(cb)
+
     @store_loaded
     def create_key_verification(self, device: OlmDevice) -> ToDeviceMessage:
         """Start a new key verification process with the given device.
