@@ -29,6 +29,7 @@ from logbook import Logger
 from .event_builders import ToDeviceMessage
 from .events import (AccountDataEvent, BadEventType, Event, InviteEvent,
                      ToDeviceEvent, EphemeralEvent)
+from .events.presence import PresenceEvent
 from .http import TransportResponse
 from .log import logger_group
 from .schemas import Schemas, validate_json
@@ -93,6 +94,8 @@ __all__ = [
     "RoomGetStateError",
     "RoomGetStateEventResponse",
     "RoomGetStateEventError",
+    "RoomGetEventResponse",
+    "RoomGetEventError",
     "RoomPutStateResponse",
     "RoomPutStateError",
     "RoomRedactResponse",
@@ -129,6 +132,10 @@ __all__ = [
     "ProfileGetAvatarError",
     "ProfileSetAvatarResponse",
     "ProfileSetAvatarError",
+    "PresenceGetResponse",
+    "PresenceGetError",
+    "PresenceSetResponse",
+    "PresenceSetError",
     "RoomKeyRequestResponse",
     "RoomKeyRequestError",
     "ThumbnailResponse",
@@ -136,7 +143,9 @@ __all__ = [
     "ToDeviceResponse",
     "ToDeviceError",
     "RoomContextResponse",
-    "RoomContextError"
+    "RoomContextError",
+    "UpdateReceiptMarkerError",
+    "UpdateReceiptMarkerResponse",
 ]
 
 
@@ -162,9 +171,9 @@ def verify(schema, error_class, pass_arguments=True):
 
 @dataclass
 class Rooms:
-    invite: Dict = field()
-    join: Dict = field()
-    leave: Dict = field()
+    invite: Dict[str, "InviteInfo"] = field()
+    join: Dict[str, "RoomInfo"] = field()
+    leave: Dict[str, "RoomInfo"] = field()
 
 
 @dataclass
@@ -198,6 +207,12 @@ class RoomSummary:
     heroes: Optional[List[str]] = None
 
 
+@dataclass
+class UnreadNotifications:
+    notification_count: Optional[int] = None
+    highlight_count: Optional[int] = None
+
+
 @dataclass
 class RoomInfo:
     timeline: Timeline = field()
@@ -205,6 +220,7 @@ class RoomInfo:
     ephemeral: List = field()
     account_data: List = field()
     summary: Optional[RoomSummary] = None
+    unread_notifications: Optional[UnreadNotifications] = None
 
     @staticmethod
     def parse_account_data(event_dict):
@@ -383,6 +399,11 @@ class RoomGetStateEventError(_ErrorWithRoomId):
     pass
 
 
+class RoomGetEventError(ErrorResponse):
+    """A response representing an unsuccessful room get event request."""
+    pass
+
+
 class RoomPutStateError(_ErrorWithRoomId):
     """A response representing an unsuccessful room state sending request."""
     pass
@@ -403,6 +424,10 @@ class RoomTypingError(_ErrorWithRoomId):
     pass
 
 
+class UpdateReceiptMarkerError(ErrorResponse):
+    pass
+
+
 class RoomReadMarkersError(_ErrorWithRoomId):
     """A response representing a unsuccessful room read markers request."""
 
@@ -528,6 +553,16 @@ class ProfileGetAvatarError(ErrorResponse):
     pass
 
 
+class PresenceGetError(ErrorResponse):
+    """Response representing a unsuccessful get presence request."""
+    pass
+
+
+class PresenceSetError(ErrorResponse):
+    """Response representing a unsuccessful set presence request."""
+    pass
+
+
 class ProfileSetAvatarError(ErrorResponse):
     pass
 
@@ -831,6 +866,31 @@ class RoomGetStateEventResponse(Response):
         return cls(parsed_dict, event_type, state_key, room_id)
 
 
+class RoomGetEventResponse(Response):
+    """A response indicating successful room get event request.
+
+    Attributes:
+        event (Event): The requested event.
+    """
+
+    event: Event = field()
+
+    @classmethod
+    @verify(
+        Schemas.room_event,
+        RoomGetEventError,
+        pass_arguments=False,
+    )
+    def from_dict(
+        cls,
+        parsed_dict: Dict[str, Any]
+    ) -> Union["RoomGetEventResponse", RoomGetEventError]:
+        event = Event.parse_event(parsed_dict)
+        resp = cls()
+        resp.event = event
+        return resp
+
+
 class RoomPutStateResponse(RoomEventIdResponse):
     """A response indicating successful sending of room state."""
     @staticmethod
@@ -977,6 +1037,12 @@ class RoomTypingResponse(_EmptyResponseWithRoomId):
         return RoomTypingError.from_dict(parsed_dict, room_id)
 
 
+class UpdateReceiptMarkerResponse(EmptyResponse):
+    @staticmethod
+    def create_error(parsed_dict):
+        return UpdateReceiptMarkerError.from_dict(parsed_dict)
+
+
 class RoomReadMarkersResponse(_EmptyResponseWithRoomId):
     """A response representing a successful room read markers request."""
 
@@ -1306,6 +1372,40 @@ class ProfileSetAvatarResponse(EmptyResponse):
         return ProfileSetAvatarError.from_dict(parsed_dict)
 
 
+@dataclass
+class PresenceGetResponse(Response):
+    """Response representing a successful get presence request.
+
+    Attributes:
+        user_id (str): The user´s id
+        presence (str): The user's presence state. One of: ["online", "offline", "unavailable"]
+        last_active_ago (int, optional): The length of time in milliseconds since an action was performed by this user.
+            None if not set.
+        currently_active (bool, optional): Whether the user is currently active. None if not set.
+        status_msg (str, optional): The state message for this user. None if not set.
+    """
+
+    user_id: str
+    presence: str
+    last_active_ago: Optional[int]
+    currently_active: Optional[bool]
+    status_msg: Optional[str]
+
+    @classmethod
+    @verify(Schemas.get_presence, PresenceGetError)
+    def from_dict(cls, parsed_dict: Dict[Any, Any], user_id: str) -> Union["PresenceGetResponse", PresenceGetError]:
+        return cls(user_id, parsed_dict.get("presence", "offline"), parsed_dict.get("last_active_ago"),
+                   parsed_dict.get("currently_active"), parsed_dict.get("status_msg"))
+
+
+class PresenceSetResponse(EmptyResponse):
+    """Response representing a successful set presence request."""
+
+    @staticmethod
+    def create_error(parsed_dict):
+        return PresenceSetError.from_dict(parsed_dict)
+
+
 @dataclass
 class ToDeviceError(ErrorResponse):
     """Response representing a unsuccessful room key request."""
@@ -1403,6 +1503,7 @@ class _SyncResponse(Response):
     device_key_count: DeviceOneTimeKeyCount = field()
     device_list: DeviceList = field()
     to_device_events: List[ToDeviceEvent] = field()
+    presence_events: List[PresenceEvent] = field()
 
     def __str__(self):
         # type: () -> str
@@ -1508,14 +1609,15 @@ class _SyncResponse(Response):
 
     @staticmethod
     def _get_join_info(
-        state_events,         # type: List[Any]
-        timeline_events,      # type: List[Any]
-        prev_batch,           # type: str
-        limited,              # type: bool
-        ephemeral_events,     # type: List[Any]
-        summary_events,       # type: Dict[str, Any]
-        account_data_events,  # type: List[Any]
-        max_events=0          # type: int
+        state_events: List[Any],
+        timeline_events: List[Any],
+        prev_batch: str,
+        limited: bool,
+        ephemeral_events: List[Any],
+        summary_events: Dict[str, Any],
+        unread_notification_events: Dict[str, Any],
+        account_data_events: List[Any],
+        max_events: int = 0,
     ):
         # type: (...) -> Tuple[RoomInfo, Optional[RoomInfo]]
         counter, state = _SyncResponse._get_room_events(
@@ -1561,9 +1663,14 @@ class _SyncResponse(Response):
             )
 
         summary = RoomSummary(
-            summary_events.get("m.invited_member_count", None),
-            summary_events.get("m.joined_member_count", None),
-            summary_events.get("m.heroes", None),
+            summary_events.get("m.invited_member_count"),
+            summary_events.get("m.joined_member_count"),
+            summary_events.get("m.heroes"),
+        )
+
+        unread_notifications = UnreadNotifications(
+            unread_notification_events.get("notification_count"),
+            unread_notification_events.get("highlight_count"),
         )
 
         account_data = RoomInfo.parse_account_data(account_data_events)
@@ -1574,18 +1681,19 @@ class _SyncResponse(Response):
             ephemeral_event_list,
             account_data,
             summary,
+            unread_notifications,
         )
 
         return join_info, unhandled_info
 
     @staticmethod
-    def _get_room_info(parsed_dict, max_events=0):
-        # type: (Dict[Any, Any], int) -> Tuple[Rooms, Dict[str, RoomInfo]]
-        joined_rooms = {
-            key: None for key in parsed_dict["join"].keys()
-        }  # type: Dict[str, Optional[RoomInfo]]
-        invited_rooms = {}  # type: Dict[str, InviteInfo]
-        left_rooms = {}     # type: Dict[str, RoomInfo]
+    def _get_room_info(
+            parsed_dict: Dict[Any, Any], max_events: int = 0,
+    ) -> Tuple[Rooms, Dict[str, RoomInfo]]:
+
+        joined_rooms: Dict[str, RoomInfo] = {}
+        invited_rooms: Dict[str, InviteInfo] = {}
+        left_rooms: Dict[str, RoomInfo] = {}
         unhandled_rooms = {}
 
         for room_id, room_dict in parsed_dict["invite"].items():
@@ -1607,6 +1715,7 @@ class _SyncResponse(Response):
                 room_dict["timeline"]["limited"],
                 room_dict["ephemeral"]["events"],
                 room_dict.get("summary", {}),
+                room_dict.get("unread_notifications", {}),
                 room_dict["account_data"]["events"],
                 max_events
             )
@@ -1618,6 +1727,14 @@ class _SyncResponse(Response):
 
         return Rooms(invited_rooms, joined_rooms, left_rooms), unhandled_rooms
 
+    @staticmethod
+    def _get_presence(parsed_dict) -> List[PresenceEvent]:
+        presence_events = []
+        for presence_dict in parsed_dict["presence"]["events"]:
+            presence_events.append(PresenceEvent.from_dict(presence_dict))
+
+        return presence_events
+
     @classmethod
     @verify(Schemas.sync, SyncError, False)
     def from_dict(
@@ -1639,6 +1756,8 @@ class _SyncResponse(Response):
             parsed_dict["device_lists"]["left"],
         )
 
+        presence_events = _SyncResponse._get_presence(parsed_dict)
+
         rooms, unhandled_rooms = _SyncResponse._get_room_info(
             parsed_dict["rooms"], max_events)
 
@@ -1649,6 +1768,7 @@ class _SyncResponse(Response):
                 key_count,
                 devices,
                 to_device,
+                presence_events,
                 unhandled_rooms,
             )
 
@@ -1658,6 +1778,7 @@ class _SyncResponse(Response):
             key_count,
             devices,
             to_device,
+            presence_events
         )
 
 
@@ -1681,6 +1802,7 @@ class PartialSyncResponse(_SyncResponse):
                 room_info.timeline.limited,
                 [],
                 {},
+                {},
                 [],
                 max_events
             )
@@ -1699,6 +1821,7 @@ class PartialSyncResponse(_SyncResponse):
                 self.device_key_count,
                 DeviceList([], []),
                 [],
+                [],
                 unhandled_rooms,
             )  # type: SyncType
         else:
@@ -1708,6 +1831,7 @@ class PartialSyncResponse(_SyncResponse):
                 self.device_key_count,
                 DeviceList([], []),
                 [],
+                []
             )
 
         if self.uuid:
