@@ -420,6 +420,13 @@ class Schemas:
                 },
                 "required": ["events"]
             },
+            "presence": {
+                "type": "object",
+                "properties": {
+                    "events": {"type": "array"}
+                },
+                "required": ["events"]
+            }
         },
         "required": [
             "next_batch",
@@ -1739,4 +1746,42 @@ class Schemas:
         ],
     }
 
+    presence = {
+        "type": "object",
+        "properties": {
+            "sender": {"type": "string"},
+            "type": {"type": "string", "const": "m.presence"},
+            "content": {
+                "type": "object",
+                "properties": {
+                    "presence": {"type": "string"},
+                    "currently_active": {"type": "boolean"},
+                    "last_active_ago": {"type": "integer"},
+                    "status_msg": {"type": "string"},
+                },
+                "required": [
+                    "presence",
+                ]
+            }
+        },
+        "required": [
+            "sender",
+            "type",
+            "content"
+        ],
+    }
+
+    get_presence = {
+        "type": "object",
+        "properties": {
+            "presence": {"type": "string"},
+            "last_active_ago": {"type": "integer"},
+            "status_msg": {"type": "string"},
+            "currently_active": {"type": "boolean"},
+        },
+        "required": [
+            "presence",
+        ],
+    }
+
     empty = {"type": "object", "properties": {}, "additionalProperties": False}
